# Magellan Autodiscovery API

## Error entity

Error responses follow this general structure and are returned as entities for
all requests that do not result in 2xx or 3xx statuses.

The error will ALWAYS contain a name and description of the error. For POST and
PATCH requests it may contain a list of errors that need to be made on the
request entity for the request to be successfull.

The error is a JSON entity:

	{
		"name": <error_name>,
		"description": <error_description>,
		"errors": [
			Error entities...
		]
	}

## Patch entity

A Patch entity follows the http://jsonpatch.com/ standard, which is specified
in RFC 6902 from the IETF.

It is an entity describing a sequence of changes to be made on a resource.

Valid operations:

* replace - Changes the value identified by *path* with the newly supplied *value*
* add - Adds the supplied *value* to a resource identified by *path*
* remove - Removes a key/value pair from the resource identified by *path*

Example patch entity:

	[
		{ "op": "replace", "path": "/baz", "value": "boo" },
		{ "op": "add", "path": "/hello", "value": ["world"] },
		{ "op": "remove", "path": "/foo"}
	]

## Job API

Manages discovery jobs that may later be executed

### GET /v1/jobs

Retrieves all existing jobs as a list

The list is a JSON encoded structure:

	[
		{
			"id": <job_id>,
			"name": <job_name>,
			"description": <job_description>,
			"method": <job_method>,
			"options": <job_options>,
			"timeout": <job_timeout>,
			"created_at": <job_create_time>,
			"updated_at": <job_update_time>
		}
		...
	]

#### Excluded IP list data

This endpoint does not return any job data for `/options/ip_list`. The IP list data is excluded because it can potentially be very large, and is typically not of immediate interest when fetching all existing jobs.

#### Responses

**200 OK** - If the list is retrieved successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

### POST /v1/jobs

Creates a new autodiscovery job based on the supplied data (as JSON) in the request body

The POST request must supply header: Content-Type: application/json

Request:

* name - string
* description - string
* method - what method to use - "seed_router", "ip_range", "ip_list"
* options - JSON object, what options to send to backend (selected by method)
	* *seed_router:*
		* seed_router - ipv4_address to seed router
		* snmp - JSON object with SNMP options
			* *SNMPv1 and SNMPv2c:*
				* version - string ("v1" or "v2c")
				* community - string (optional, defaults to "public")
			* *SNMPv3:*
				* version - string ("v3")
				* secname - string
				* authpass - string (optional, unless privpass is given)
				* privpass - string (optional)
	* *ip_range:*
		* ip_range - ip range/CIDR string
	* *ip_list:*
		* ip_list - string with ip addresses or ranges separated by newline or space characters.
* timeout - int - Timeout in seconds before a job execution should be automatically cancelled. Optional, defaults to 3600 seconds (1 hour).

#### Example

Job that performs a seed router discovery from 172.27.87.120.

	{
		"name": "my-job",
		"description": "This job discovers my favorite network",
		"method": "seed_router",
		"options": {
			"seed_router": "172.27.87.120",
			"snmp": {
				"version": "v2c",
				"community": "public"
			}
		}
	}

#### Responses

**201 Create** - On successfull job creation, should return an entity
consisting of the URL to the newly created job in addition to supplying the
Location header with a URL to the job endpoint.

**400 Bad Request** - If the request is not valid, such as when the entity
sent does not contain all required fields or when a field fails validation.
The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

**409 Conflict** - If the name of the new job already exists. The response entity should be an **Error**.

**415 Unsupported Media Type** - If the content-type header is not application/json.
The response entity should be an **Error**.

### GET /v1/jobs/&lt;job_id&gt;

Retrieves a specific job with a given job_id (UUID)

The job is a JSON encoded structure:

	{
		"id": <job_id>,
		"name": <job_name>,
		"description": <job_description>,
		"method": <job_method>,
		"options": <job_options>,
		"timeout": <job_timeout>,
		"created_at": <job_create_time>,
		"updated_at": <job_update_time>
	}

#### Responses

**200 OK** - If the job is retrieved successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied job_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

### PATCH /v1/jobs/&lt;job_id&gt;

Updates an existing autodiscovery job based on the supplied data (as JSON) in
the request body. The entity should be a Patch entity, a list of operations to
perform on the resource.

The PATCH request must supply header: Content-Type: application/json

Fields available for patching:

* name - string
* description - string
* method - string
* options - object
* timeout - integer

The entity is a JSON encoded structure:

	[
		{"op": <patch_operation>, "path": <patch_field>, "value": <patch_value>}
		...
	]

Example:

	[
		{"op": "replace", "path": "/name", "value": "New job name"}
	]

#### Responses

**200 OK** - If the job was updated successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied job_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

**409 Conflict** - If the update attempts to give the job the same name as that of an another existing job. The response entity should be an **Error**.

**415 Unsupported Media Type** - If the content-type header is not application/json. The response entity should be an **Error**.

### DELETE /v1/jobs/&lt;job_id&gt;

Removes a job from the database

#### Responses

**204 No Content** - If the job was deleted successfully. The response contains no body.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied job_id does not exist. The response entity should be an **Error**.

### GET /v1/jobs/&lt;job_id&gt;/executions

Retrieves all existing executions for a job_id as a list

The list is a JSON encoded structure:

	[
		{
			"id": <execution_id>,
			"job_id": <job_id>,
			"name": <execution_job_name>,
			"description": <execution_job_description>,
			"status": <execution_job_status>,
			"created_at": <execution_job_create_time>,
			"updated_at": <execution_job_update_time>,
			"complete_time": <execution_job_complete_time>,
		}
		...
	]

#### Responses

**200 OK** - If the list is retrieved successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied job_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

## Execution API

Manages discovery jobs executions

### GET /v1/executions

Retrieves all existing executions as a list

The list is a JSON encoded structure:

	[
		{
			"id": <execution_id>,
			"job_id": <job_id>,
			"name": <execution_job_name>,
			"description": <execution_job_description>,
			"status": <execution_job_status>,
			"created_at": <execution_job_create_time>,
			"updated_at": <execution_job_update_time>,
			"complete_time": <execution_job_complete_time>
		}
		...
	]

#### Responses

**200 OK** - If the list is retrieved successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

### POST /v1/executions

Creates a new autodiscovery job execution based on the supplied data (as JSON) in the request body.

The execution of the job will be done as soon as possible.

The POST request must supply header: Content-Type: application/json

Request:

* job_id - string
* description - string
* name - string

#### Example

Start autodiscovery job execution based on job_id definition.

	{
		"job_id": "78cc910c-1102-40ee-af99-d7e416f54c0a",
		"name": "Bulk rescan",
		"description": "Provisioned 20 new hosts, rescanning",
	}

#### Responses

**201 Create** - On successful job execution creation, should return an entity
consisting of the URL to the newly created job execution in addition to supplying the
Location header with a URL to the job execution endpoint.

**400 Bad Request** - If the request is not valid, such as when the entity
sent does not contain all required fields or when a field fails validation.
The response entity should be an **Error**.

**404 Not Found** - If the supplied job_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

**415 Unsupported Media Type** - If the content-type header is not application/json. The response entity should be an **Error**.

**503 Service Unavailable** - If there already is a running execution associated with the supplied job_id. The response entity should be an **Error**.

### GET /v1/executions/&lt;execution_id&gt;

Retrieves all information about an execution

The entity is a JSON encoded structure:

	{
		"id": <execution_id>,
		"job_id": <job_id>,
		"name": <execution_job_name>,
		"description": <execution_job_description>,
		"status": <execution_job_status>,
		"created_at": <execution_job_create_time>,
		"updated_at": <execution_job_update_time>,
		"complete_time": <execution_job_complete_time>
	}

#### Responses

**200 OK** - If the entity is retrieved successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied execution_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

### PATCH /v1/executions/&lt;execution_id&gt;

Updates an existing autodiscovery execution based on the supplied data (as
JSON) in the request body. The entity should be a Patch entity, a list of
operations to perform on the resource.

The PATCH request must supply header: Content-Type: application/json

Fields available for patching:

* name - string
* description - string
* status - string
* schedule_status - object
* username - integer

The entity is a JSON encoded structure:

	[
		{"op": <patch_operation>, "path": <patch_field>, "value": <patch_value>}
		...
	]

Example:

	[
		{"op": "replace", "path": "/name", "value": "New execution name"}
	]

#### Responses

**200 OK** - If the execution was updated successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied execution_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

**409 Conflict** - If the update attempts to give the execution the same id as that of an another existing execution. The response entity should be an **Error**.

**415 Unsupported Media Type** - If the content-type header is not application/json. The response entity should be an **Error**.

### DELETE /v1/executions/&lt;execution_id&gt;

Removes an execution job from the database

#### Responses

**204 No Content** - If the job was deleted successfully. The response contains no body.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied execution_id does not exist. The response entity should be an **Error**.

### PUT /v1/executions/cancel/&lt;execution_id&gt;

Cancels the ongoing discovery process of an execution job

#### Responses

**202 Accepted** - If the attempt to cancel the execution has been accepted. This response will not wait for the execution to actually stop.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied execution_id does not exist. The response entity should be an **Error**.

### GET /v1/executions/&lt;execution_id&gt;/devices

Retrieves all devices found by an execution

The entity is a JSON encoded structure:

	[
		{
			"address": <device_ip_address>,
			"details": <device_details>,
			"neighbours": [<device_ip_addresses>],
		},
		...
	]

#### Responses

**200 OK** - If the entity is retrieved successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied execution_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

### PATCH /v1/executions/&lt;execution_id&gt;/devices/&lt;device_id&gt;

Updates a discovered device based on the supplied patch in the request body. The entity should be a Patch entity, a list of operations to perform on the resource.

This method should typically only be used to set the config field of a device, but could theoretically change any field on the device.

Example:

	[
		{"op": "replace", "path": "/config", "value": {"hostgroup": "MyHostgroup"}}
	]

#### Responses

**200 OK** - If the device was updated successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied device_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

**409 Conflict** - If the update attempts to give the device the same name as that of an another existing job. The response entity should be an **Error**.

**415 Unsupported Media Type** - If the content-type header is not application/json. The response entity should be an **Error**.

## Exclusion list API

Manages the lists of excluded IP addresses, ranges and subnets.

### GET /v1/exclusion-lists

Retrieves all existing exclusion lists as a list.

The entity is a JSON encoded structure:

	[
		{
			"id": <exclusion_list_id>,
			"name": <exclusion_list_name>,
			"description": <exclusion_list_description>,
			"created_at": <creation_time>,
			"updated_at": <last_update_time>,
			"items": [
				{
					"id": <exclusion_list_item_id>,
					"exclusion_list_id": <exclusion_list_id>,
					"address": <ip_address_or_range>,
					"comment": <comment>,
					"created_at": <creation_time>,
					"updated_at": <last_update_time>,
				},
				...
			]
		},
		...
	]

#### Responses

**200 OK** - If the entity is retrieved successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

### POST /v1/exclusion-lists

Creates a new exclusion list based on the supplied data (as JSON) in the request body.

The POST request must supply header: Content-Type: application/json

Request:

* name - string
* description (optional) - string
* items - list of exclusion list items:
	* address - string
	* comment (optional) - string

#### Example

	{
		"name": "My exclusion list",
		"description": "Exclude all those things I don't want to find",
		"items": [
			{
				"address": "192.168.100.100",
				"comment": "Hidden server"
			},
			{
				"address": "172.56.0.0/16",
				"comment": "Hidden network"
			}
		]
	}

#### Responses

**201 Create** - On successful exclusion list creation, should return an entity consisting of the new exclusion list entity in addition to supplying the Location header with a URL to the exclusion list endpoint.

**400 Bad Request** - If the request is not valid, such as when the entity sent does not contain all required fields or when a field fails validation. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

**415 Unsupported Media Type** - If the content-type header is not application/json. The response entity should be an **Error**.

### GET /v1/exclusion-lists/&lt;exclusion_list_id&gt;

Retrieves a specific exclusion list with a given exclusion_list_id (UUID).

The exclusion list is a JSON encoded structure:

	{
		"id": <exclusion_list_id>,
		"name": <exclusion_list_name>,
		"description": <exclusion_list_description>,
		"created_at": <creation_time>,
		"updated_at": <last_update_time>,
		"items": [
			{
				"id": <exclusion_list_item_id>,
				"exclusion_list_id": <exclusion_list_id>,
				"address": <ip_address_or_range>,
				"comment": <comment>,
				"created_at": <creation_time>,
				"updated_at": <last_update_time>,
			},
			...
		]
	}

#### Responses

**200 OK** - If the exclusion list is retrieved successfully.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied exclusion_list_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

### GET /v1/exclusion-lists/default

Retrieves the special *default* exclusion list. If the default exclusion list does not exist it will be created. The response and errors are the same as for **GET /v1/exclusion-lists/&lt;exclusion_list_id&gt;** above.

#### Deprecated

The default endpoint will probably be removed in the future when the Autodiscovery UI gains support for multiple exclusion lists.

### DELETE /v1/exclusion-lists/&lt;exclusion_list_id&gt;

Removes a exclusion list from the database.

#### Responses

**204 No Content** - If the exclusion list was deleted successfully. The response contains no body.

**400 Bad Request** - If the request is not valid. The response entity should be an **Error**.

**404 Not Found** - If the supplied exclusion_list_id does not exist. The response entity should be an **Error**.

### PUT /v1/exclusion-lists/&lt;exclusion_list_id&gt;

Replaces a specific exclusion list with a given exclusion_list_id (UUID) with the supplied data (as JSON) in the request body.

The PUT request must supply header: Content-Type: application/json

Request:

* name - string
* description (optional) - string
* items - list of exclusion list items:
	* address - string
	* comment (optional) - string

#### Example

	{
		"name": "My exclusion list",
		"description": "Exclude all those things I don't want to find",
		"items": [
			{
				"address": "192.168.100.100",
				"comment": "Hidden server"
			},
			{
				"address": "172.56.0.0/16",
				"comment": "Hidden network"
			}
		]
	}

#### Responses

**200 OK** - On successful update of the exclusion list, should return an entity consisting of the updated exclusion list entity in addition to supplying the Location header with a URL to the exclusion list endpoint.

**400 Bad Request** - If the request is not valid, such as when the entity sent does not contain all required fields or when a field fails validation. The response entity should be an **Error**.

**404 Not Found** - If the supplied exclusion_list_id does not exist. The response entity should be an **Error**.

**406 Not Acceptable** - If the request has an Accept header and it is not equal to application/json. The response entity should be an **Error**.

**415 Unsupported Media Type** - If the content-type header is not application/json. The response entity should be an **Error**.
